# Management Network Upgrade CSM 1.2 to 1.3

- [Prerequisites](#prerequisites)
- [Backup switch configurations](#backup-switch-configurations)
- [Warnings](#warnings)
- [Mellanox](#mellanox)
- [Dell](#dell)
- [Aruba Spine](#aruba-spine)
- [Aruba Leaf and Leaf BMC](#aruba-leaf-and-leaf-bmc)
- [Run network tests](#run-network-tests)

## Prerequisites

- System is currently running CANU-generated CSM 1.2 configurations.
- Switch configurations have been generated for CSM 1.3.
  - See [Generate Switch Configurations](generate_switch_configs.md).
  - Ensure that site [Custom Configurations](https://github.com/Cray-HPE/canu/#generate-switch-configs-including-custom-configurations) used during installation have been retrieved or recreated.
- CANU is installed with version 1.6.7 or greater.
  - Run `canu --version` to see version.
  - Update the CANU RPM from the release tarball.
    - See [Update CANU From CSM Tarball](canu/update_canu_from_csm_tarball.md).
  - Alternatively, upgrade or install the latest version of CANU from GitHub.
    - See [Install/Upgrade CANU](canu_install_update.md).

For this procedure, log in to `ncn-m001` as an administrative user. The log in should be over the site connection (`lan0`), not the CMN, to prevent loss of connectivity to the system.

## Backup switch configurations

1. (`ncn-m001#`) Use CANU to backup the switch running configurations.

    Enter the switch administrative password when prompted.

     ```bash
     mkdir switch-upgrades-csm-1.3
     cd switch-upgrades-csm-1.3
     canu backup network --folder 1.2
     ```

1. (`ncn-m001#`) Stage the CANU-generated CSM 1.3 switch configurations in the `switch-upgrades-csm-1.3/1.3` directory.

## Warnings

Understanding the switch configuration changes is critical. The following configurations risk a network outage, if not applied correctly:

- Generating switch configuration without preserving site-specific values (by using the `--custom-configuration` flag).
- Changes to ISL (MAGP, VSX, etc.) configurations.
- Changes to Spanning Tree.
- Changes to ACLs or ACL ordering.
- Changes to VRFs.
- Changes to default route.
- Changes to MLAG/LACP.
  
## Mellanox

1. (`ncn-m001#`) Use CANU to see the differences between the 1.2 and 1.3 switch configurations.

     ```bash
     canu validate switch config --running ./1.2/sw-spine-002.cfg --generated ./1.3/sw-spine-002.cfg --vendor mellanox
     ```

1. Apply the changed switch configurations.

    CANU remediation configurations are not supported on Mellanox. Switch changes must be manually applied on the switch based on the configuration differences.

1. (`switch#`) Enter configuration mode and show the banner.

   ```text
   enable
   configure terminal
   show banner
   ```

   Example output of the `show banner` command:

   ```text
   Banners:
     Message of the Day (MOTD):

       ###############################################################################
       # CSM version:  1.3
       # CANU version: 1.6.7
       ###############################################################################

     Login:
       NVIDIA Onyx Switch Management

     Logout:
   ```

1. (`switch#`) Save the configuration based on the banner output.

   ```text
   configuration write to csm1.3-canu1.6.5
   ```

## Dell

1. (`ncn-m001#`) Use CANU to see the differences between the 1.2 and 1.3 switch configurations.

     ```bash
     canu validate switch config --running 1.2/sw-leaf-bmc-001.cfg --generated 1.3/sw-leaf-bmc-001.cfg --vendor dell --remediation
     ```

1. Apply the remediation switch configurations.

1. (`switch#`) Save the configuration.

   ```text
   copy config://startup.xml config://csm1.2-canu1.6.5
   ```

   Example output:

   ```text
   Copy completed
   ```

## Aruba Spine

1. (`ncn-m001#`) Use CANU to see the differences between the 1.2 and 1.3 switch configurations.

    ```bash
    canu validate switch config --running ./1.2/sw-spine-002.cfg --generated ./1.3/sw-spine-002.cfg --vendor aruba --remediation
    ```

1. Apply the remediation switch configurations.

    Do this in two stages:

    1. Apply just the `prefix-list` and `route-maps`.

    1. Apply the remaining configuration.

1. (`switch#`) Show the configuration version.

   ```text
   configure terminal
   show banner motd
   ```

   Example output of the `show banner motd` command:

   ```text
   ###############################################################################
   # CSM version:  1.3
   # CANU version: 1.6.7
   ###############################################################################
   ```

1. (`switch#`) Save the configuration.

   ```text
   write memory
   copy running-config checkpoint CSM1_3_CANU_1_6_7
   ```

## Aruba Leaf and Leaf BMC

1. (`ncn-m001#`) Use CANU to see the differences between the 1.2 and 1.3 switch configurations.

   ```bash
   canu validate switch config --running surtur/1.2/sw-leaf-bmc-001.cfg --generated surtur/1.3/sw-leaf-bmc-001.cfg --vendor aruba --remediation
   ```

1. Apply the remediation switch configurations.

    Do this in two stages:

    1. Apply just the `prefix-list` and `route-maps`.

    1. Apply the remaining configuration.

1. (`switch#`) Show the configuration version.

   ```text
   configure terminal
   show banner motd
   ```

   Example output of the `show banner motd` command:

   ```text
   ###############################################################################
   # CSM version:  1.3
   # CANU version: 1.6.7
   ###############################################################################
   ```

1. (`switch#`) Save the the configuration.

   ```text
   write memory
   copy running-config checkpoint CSM1_3_CANU_1_6_7
   ```

## Run network tests

1. (`ncn-m001#`) Use CANU to test the network.

   ```bash
   canu test network --csm 1.3
   ```
