# Aruba Installation and Configuration Guide

This documentation helps network administrators and support personnel install and manage Aruba network devices in a CSM install.

The HPE Cray recommended way of configuring the network is by using the CANU tool. Therefore this guide will not go into detail on how to configure
each switch manually using the CLI. Instead, it will give helpful examples of how to configure/use features generated by CANU, in order to provide
administrators easy ways to customize their installation.

Also included in this guide are the current documented and supported network scenarios.

**`NOTE`** Not every configuration option is covered here; for any configuration outside of the scope of this document, refer to the official
Aruba user manual. See [Aruba Networks](https://asp.arubanetworks.com/).

This document is intended for network administrators and support personnel.

**`NOTE`** The display and command lines illustrated in this document are examples and might not exactly match any particular environment. The switch
and accessory drawings in this document are for illustration only, and may not exactly match installed products.

## CANU

See [CSM Automatic Network Utility (CANU)](../canu/README.md)

## Examples of Network Topologies

* [Very Large](network_topologies.md#very-large)
* [Large](network_topologies.md#large)
* [Medium](network_topologies.md#medium)
* [Small](network_topologies.md#small)

## Network Design Explained

* [What is Spine-Leaf Architecture?](spine_leaf_architecture.md)
* [How Does a Spine-Leaf Architecture Differ From Traditional Network Designs?](spine_leaf_architecture.md)
* [Why are Spine-Leaf Architectures Becoming More Popular?](spine_leaf_architecture.md)
* [What is VSX?](vsx_architecture.md)
* [What are the benefits of VSX?](vsx_architecture.md)
* [VSX summary](vsx_architecture.md)

## Built in High Availability

* [VSX: MCLAG link HA](mclag_link_ha.md)
* [VSX: ISL HA](mclag_isl_ha.md)
* [VSX: Split](mclag_split.md)
* [VSX: Member Power Failure](mclag_power_failure.md)

## Management Network Overview

* [Network Types – Naming and Segment Function](network_naming_function.md)
* [Network Traffic Pattern](network_traffic_pattern.md)
* [System Management Network Functions](management_network_function_in_detail.md)

## Key Features Used in the Management Network Configuration

* [Key Feature List](key_features.md)
* [Typical Configuration of VSX](typical_VSX_configuration.md)
* [Typical Configuration of MCLAG Link](typical_mclag_port_configuration.md)
* [Typical Edge Port Configuration](typical_edge_port_configuration.md)

## How to Connect Management Network to a Campus Network

* [Connect the Management Network to a Campus Network](requirements_and_optional_configuration.md)
* [Scenario A: Network Connection via Management Network](scenario-a.md)
* [Scenario B: Network Connection via High-Speed Network](scenario-b.md)
* [Example of How to Configure Scenario A or B](management_network_configuration_example.md)

## Manage Switches from CLI

### Device Management

* [Management Interface](management_interface.md)
* [Network Time Protocol (NTP) Client](ntp.md)
* [Domain Name System (DNS) Client](dns-client.md)
* [Message-Of-The-Day (MOTD)](motd.md)
* [Exec Banners](exec_banner.md)
* [Hostname](hostname.md)
* [Domain Name](domain_name.md)
* [Secure Shell (SSH)](ssh.md)
* [Remote Logging](remote_logging.md)
* [Web User Interface (Web UI)](web-ui.md)
* [Simple Network Management Protocol (SNMP) Agent](snmp-agent.md)
* [SNMPv2c Community](snmp-community.md)
* [SNMP Traps](snmp_trap.md)
* [SNMPv3 Users](snmpv3_users.md)
* [Bluetooth Capabilities](bluetooth.md)

### Layer One Features

* [Physical Interfaces](physical_interfaces.md)
* [Redundant Power Supplies](redundant_power_supplies.md)
* [Locator LED](locator_led.md)
* [Cable Diagnostics](cable_diagnostics.md)

### Layer Two Features

* [Unidirectional Link Detection (UDLD)](udld.md)
* [Link Layer Discovery Protocol (LLDP)](lldp.md)
* [Virtual Local Access Networks (VLANs)](vlan.md)
* [Native VLAN](native_vlan.md)
* [VLAN Trunking 802.1Q](vlan_trunking_8021q.md)
* [Link Aggregation Group (LAG)](lag.md)
* [Virtual Switching Extension (VSX)](vsx.md)
* [Multi-Chassis Link Aggregation Group (MCLAG)](mlag.md)
* [VSX Sync](vsx_sync.md)
* [Virtual Switching Framework (VSF) 6300 Only](vsf.md)
* [Multiple Spanning Tree Protocol (MSTP)](mstp.md)

### Layer Three Features

* [Routed Interfaces](routed_interface.md)
* [VLAN Interface](vlan_interface.md)
* [Address Resolution Protocol (ARP)](arp.md)
* [Static Routing](static_routing.md)
* [Loopback Interface](loopback.md)
* [Open Shortest Path First (OSPF) v2](ospfv2.md)
* [BGP Basics](bgp_basic.md)

### Multicast

* [IGMP](igmp.md)
* [MSDP](msdp.md)
* [PIM-SM Bootstrap Router (BSR) and Rendezvous-Point (RP)](pim.md)

### Security

* [Access Control Lists (ACLs)](acl.md)
* [TACACS](tacacs.md)
* [RADIUS](radius.md)
* [Port Security](port_security.md)
* [802.1X](8021x.md)
* [MAC Authentication](mac_auth.md)

### Quality of Service

* [Initial Prioritization)](initial_prioritization.md)
* [Classifier Policies](classifier_policies.md)
* [Queuing and Scheduling](queuing_and_scheduling.md)

### Perform a VSX Upgrade on Aruba Switches

* [VSX Upgrade](upgrade.md)

### Switch Replacement in the VSX Cluster

* [Switch replacement in the VSX Cluster](vsx_switch_replacement.md)

### Back Up Switch Configuration

* [Backing up switch configuration](backup.md)

### Erase All zeroize

* [Erase all zeroize](zeroize.md)

## Troubleshooting

### Port Mirroring on Aruba CX

* [Port Mirroring](port_mirroring.md)

### DHCP

* [Confirm the status of the `cray-dhcp-kea` pods/services](status_of_cray-dhcp-kea_pods.md)
* [Check current DHCP leases](check_current_dhcp_leases.md)
* [Check HSM](check_hsm.md)
* [Check Kea DHPC logs](check_kea_dhcp_logs.md)
* [TCPDUMP](ncn_tcpdump.md)
* [Check BGP and MetalLB](check_bgp_and_metallb.md)
* [Getting incorrect IP address. Duplicate IP address check](duplicate_ip.md)
* [Large number of DHCP declines during a node boot](dhcp_decline.md)

### DNS

### PXE Boot

* [NCNs on install](ncns_on_install.md)
* [Rebooting NCN and PXE fails](reboot_pxe_fail.md)
* [Verify BGP](verify_bgp.md)
* [Verify route to TFTP](verify_route_to_tftp.md)
* [Test TFTP traffic (Aruba Only)](test_tftp_traffic.md)
* [Check DHCP lease is getting allocated](check_dhcp_lease_is_getting_allocated.md)
* [Verify DHCP traffic on workers](verify_dhcp_traffic_on_workers.md)
* [Verify switches are forwarding DHCP traffic](verify-switches_are_forwarding_dhcp_traffic.md)
* [Computes/UANs/Application Nodes](compute_uan_application_nodes.md)
